# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.network.v2 import _base
from openstack import resource


class SecurityGroupsDefaultStatefulness(_base.NetworkResource):
    resource_key = 'security_group_default_statefulness'
    resources_key = 'security_groups_default_statefulness'
    base_path = '/security-groups-default-statefulness'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id',
        'project_id',
        'stateful',
        'sort_dir',
        'sort_key',
    )

    # Properties
    #: The project to which this setting applies.
    #: ``None`` means system-wide.
    project_id = resource.Body('project_id')
    #: The default value for the ``stateful`` attribute when creating
    #: new security groups.
    #: *Type: bool*
    stateful = resource.Body('stateful', type=bool)
