# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Mapping(resource.Resource):
    resource_key = 'mapping'
    resources_key = 'mappings'
    base_path = '/OS-FEDERATION/mappings'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    create_opts = resource.CreateOpts(method='PUT')
    commit_method = 'PATCH'

    _query_mapping = resource.QueryParameters()

    # Properties
    #: The rules of this mapping. *Type: list*
    rules = resource.Body('rules', type=list)
    #: The attribute mapping schema version. *Type: string*
    schema_version = resource.Body('schema_version', type=str)

    #: The identifier of the mapping. *Type: string*
    name = resource.Body('id')
