# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.functional.shared_file_system.v2 import base


class QuotaSetTest(base.BaseSharedFileSystemTest):
    def test_quota_sets(self):
        self.project_id = self.operator_cloud.current_project_id
        assert self.project_id is not None
        default_quota_set = self.operator_cloud.share.get_quota_set(
            self.project_id
        )
        self.assertIsNotNone(default_quota_set)
        self.assertIsNotNone(default_quota_set.id)
